import { __read, __spread, __values } from "tslib";
import { isObservable, Observable, of, Subject } from 'rxjs';
import { switchMap } from 'rxjs/operators';
/**
 * Operator to split a stream of Objects into it's individual property streams
 *
 * Source:
 * ---{ a:1, b:42 }-----{ a: 2, c: 1 }----{ b: 1337 }-------|
 *
 *
 * ---key:a,key:b-------key:c-------------key:b-------------|
 *
 *    key:a
 *    1-----------------2-----------------------------------|
 *
 *    key:b                               key:b
 *    42----------------|                 42----------------|
 *
 *                      key:c
 *                      1-----------------------------------|
 */
export function splitPropsToStreams() {
    return function (source$) {
        return new Observable(function (observer) {
            var propSubjectRegistry = new Map();
            var subscription = source$.subscribe({
                next: function (change) {
                    var e_1, _a, e_2, _b;
                    var changeEntries = Object.entries(change);
                    try {
                        for (var changeEntries_1 = __values(changeEntries), changeEntries_1_1 = changeEntries_1.next(); !changeEntries_1_1.done; changeEntries_1_1 = changeEntries_1.next()) {
                            var _c = __read(changeEntries_1_1.value, 2), key = _c[0], value = _c[1];
                            var stream = void 0;
                            if (!propSubjectRegistry.has(key)) {
                                stream = new Subject();
                                propSubjectRegistry.set(key, stream);
                                var propStream = stream.pipe(switchMap(function (o) { return o; }));
                                propStream['key'] = key;
                                observer.next(propStream);
                            }
                            if (!stream) {
                                stream = propSubjectRegistry.get(key);
                            }
                            stream.next(isObservable(value) ? value : of(value));
                        }
                    }
                    catch (e_1_1) { e_1 = { error: e_1_1 }; }
                    finally {
                        try {
                            if (changeEntries_1_1 && !changeEntries_1_1.done && (_a = changeEntries_1.return)) _a.call(changeEntries_1);
                        }
                        finally { if (e_1) throw e_1.error; }
                    }
                    try {
                        for (var _d = __values(propSubjectRegistry.keys()), _e = _d.next(); !_e.done; _e = _d.next()) {
                            var oldKey = _e.value;
                            if (oldKey in change) {
                                continue;
                            }
                            var deprecatedStream = propSubjectRegistry.get(oldKey);
                            deprecatedStream.complete();
                            propSubjectRegistry.delete(oldKey);
                        }
                    }
                    catch (e_2_1) { e_2 = { error: e_2_1 }; }
                    finally {
                        try {
                            if (_e && !_e.done && (_b = _d.return)) _b.call(_d);
                        }
                        finally { if (e_2) throw e_2.error; }
                    }
                },
                error: function (e) { return observer.error(e); },
                complete: function () { return observer.complete(); },
            });
            // complete all streams on unsubscription
            subscription.add(function () {
                __spread(propSubjectRegistry.values()).forEach(function (v) { return v.complete(); });
            });
            return subscription;
        });
    };
}
